"""Python library to connect deCONZ and Home Assistant to work together."""

from . import LightBase


class ConfigurationTool(LightBase):
    """deCONZ hardware antenna."""

    ZHATYPE = ("Configuration tool",)
