"""Python library to connect deCONZ and Home Assistant to work together."""

# Legacy imports kept for backwards compatibility

from .interfaces.groups import *  # noqa: F401, F403
from .models.group import *  # noqa: F401, F403
